/* Algoritmul lui Lee
 * - Algoritm de Programare Dinamica
 * 
 * Problema Labirintului
 * Intr-un labirint de dimensiuni n x m, celulele libere
 * sunt marcate cu '_', iar celulele ocupate cu '#'.
 * Un soricel porneste de la poz (ip, jp) si trebuie sa ajunga
 * la branza, adica in celula (is, js).
 * 
 * Sa se determine numarul minim de pasi, pe care soricelul trebuie sa-i faca
 * de la (ip, jp) la (is, js).
 * 
 * Deplasarea se face din celula curenta, intr-un dintre celule adiacente.
 * 
 * 
 * 
 */ 

#include <fstream>
#include <algorithm>
#include <iomanip>
using namespace std;

ifstream fin("labirint.in");
ofstream fout("labirint.out");

const int Dim = 101, Inf = 1e9; 
const int di[] = {-1, 0, 1, 0};   // variatiile de pozitii la deplasari spre N, E, S, V
const int dj[] = {0, 1, 0, -1}; 

char L[Dim][Dim];
int c[Dim][Dim];    // c[i][j] - distanta minima de la (ip, jp) la 
                    // celula (i, j)
int n, m;
int ip, jp, is, js; // poz de plecare, respectiv sosire

void CitesteDate();
void Lee();
void Debug();
bool Ok(int i, int j);

int main()
{
	CitesteDate();
	Lee();
	Debug();
	fout << c[is][js];
}

void Lee()
{
	for (int i = 1; i <= n; ++i)
		for (int j = 1; j <= m; ++j)
			c[i][j] = Inf;
			
	c[ip][jp] = 0;
	
	int pas = 0;
	bool modif = true;
	int iv, jv;
	while (modif)   // O(n*m/2 * n*m) = O(n^4)
	{
		modif = false; // presupunem ca nu vor mai fi modificari in matrice
		for (int i = 1; i <= n; ++i)
			for (int j = 1; j <= m; ++j)
				if (c[i][j] == pas)
					for (int d = 0; d < 4; ++d)
					{
						iv = i + di[d];
						jv = j + dj[d];
						if (Ok(iv, jv) && c[iv][jv] > c[i][j] + 1)
						{
							c[iv][jv] = c[i][j] + 1;
							modif = true;
						}
					}
		pas++;
	}	
}

bool Ok(int i, int j)
{
	if (i < 1 || i > n || j < 1 || j > m) // in afara matricei
		return false;
	if (L[i][j] == '#') // obstacol
		return false;
		
	return true;
}


void Debug()
{
	for (int i = 1; i <= n; ++i)
	{
		for (int j = 1; j <= m; ++j)
			if (L[i][j] == '#')
				fout << " []";
			else
				if (c[i][j] != Inf)
					fout << setw(3) << c[i][j];
				else
					fout << " oo";
		fout << '\n';
	}
	
	fout << '\n';
}

void CitesteDate()
{
	fin >> n >> m;
	fin >> ip >> jp >> is >> js;
	
	for (int i = 1; i <= n; ++i)
		for (int j = 1; j <= m; ++j)
			fin >> L[i][j];
}



